@@ -21,6 +21,7 @@
 #include <cstdint>
 
 #include "absl/status/statusor.h"
+#include "src/core/channelz/property_list.h"
 #include "src/core/util/bitset.h"
 
 namespace grpc_core {
@@ -29,65 +30,47 @@ namespace chaotic_good {
 // Remember to add new frame types to frame_fuzzer.cc
 enum class FrameType : uint8_t {
   kSettings = 0x00,
+  kTcpSecurityFrame = 0x01,  // For TcpFrameTransport
   kClientInitialMetadata = 0x80,
   kClientEndOfStream = 0x81,
   kServerInitialMetadata = 0x91,
   kServerTrailingMetadata = 0x92,
   kMessage = 0xa0,
+  kBeginMessage = 0xa1,
+  kMessageChunk = 0xa2,
   kCancel = 0xff,
 };
 
+std::string FrameTypeString(FrameType type);
+
 inline std::ostream& operator<<(std::ostream& out, FrameType type) {
-  switch (type) {
-    case FrameType::kSettings:
-      return out << "Settings";
-    case FrameType::kClientInitialMetadata:
-      return out << "ClientInitialMetadata";
-    case FrameType::kClientEndOfStream:
-      return out << "ClientEndOfStream";
-    case FrameType::kMessage:
-      return out << "Message";
-    case FrameType::kServerInitialMetadata:
-      return out << "ServerInitialMetadata";
-    case FrameType::kServerTrailingMetadata:
-      return out << "ServerTrailingMetadata";
-    case FrameType::kCancel:
-      return out << "Cancel";
-    default:
-      return out << "Unknown[" << static_cast<int>(type) << "]";
-  }
+  return out << FrameTypeString(type);
+}
+
+template <typename Sink>
+void AbslStringify(Sink& sink, FrameType type) {
+  sink.Append(FrameTypeString(type));
 }
 
 struct FrameHeader {
   FrameType type = FrameType::kCancel;
-  uint16_t payload_connection_id = 0;
   uint32_t stream_id = 0;
   uint32_t payload_length = 0;
 
-  // Parses a frame header from a buffer of 12 bytes. All 12 bytes are consumed.
-  static absl::StatusOr<FrameHeader> Parse(const uint8_t* data);
-  // Serializes a frame header into a buffer of 12 bytes.
-  void Serialize(uint8_t* data) const;
   // Report contents as a string
   std::string ToString() const;
-  // Required padding to maintain alignment.
-  uint32_t Padding(uint32_t alignment) const {
-    if (payload_connection_id == 0) {
-      return 0;
-    }
-    if (payload_length % alignment == 0) {
-      return 0;
-    }
-    return alignment - (payload_length % alignment);
-  }
 
   bool operator==(const FrameHeader& h) const {
     return type == h.type && stream_id == h.stream_id &&
-           payload_connection_id == h.payload_connection_id &&
            payload_length == h.payload_length;
   }
-  // Frame header size is fixed to 12 bytes.
-  enum { kFrameHeaderSize = 12 };
+
+  channelz::PropertyList ChannelzProperties() const {
+    return channelz::PropertyList()
+        .Set("type", FrameTypeString(type))
+        .Set("stream_id", stream_id)
+        .Set("payload_length", payload_length);
+  }
 };
 
 inline std::ostream& operator<<(std::ostream& out, const FrameHeader& h) {
