@@ -24,7 +24,6 @@
 #include <tuple>
 
 #include "absl/cleanup/cleanup.h"
-#include "absl/log/check.h"
 #include "absl/log/log.h"
 #include "absl/random/bit_gen_ref.h"
 #include "absl/random/random.h"
@@ -47,6 +46,7 @@
 #include "src/core/lib/slice/slice_buffer.h"
 #include "src/core/lib/transport/promise_endpoint.h"
 #include "src/core/telemetry/metrics.h"
+#include "src/core/util/grpc_check.h"
 #include "src/core/util/ref_counted_ptr.h"
 
 namespace grpc_core {
@@ -218,7 +218,7 @@ auto ChaoticGoodServerTransport::CallOutboundLoop(
 absl::Status ChaoticGoodServerTransport::NewStream(
     ChaoticGoodTransport& transport, const FrameHeader& header,
     SliceBuffer payload) {
-  CHECK_EQ(header.payload_length, payload.Length());
+  GRPC_CHECK_EQ(header.payload_length, payload.Length());
   auto client_initial_metadata_frame =
       transport.DeserializeFrame<ClientInitialMetadataFrame>(
           header, std::move(payload));
@@ -342,11 +342,21 @@ ChaoticGoodServerTransport::ChaoticGoodServerTransport(
           args.GetObjectRef<grpc_event_engine::experimental::EventEngine>()),
       outgoing_frames_(4),
       message_chunker_(config.MakeMessageChunker()) {
+  std::string peer_string =
+      grpc_event_engine::experimental::ResolvedAddressToString(
+          control_endpoint.GetPeerAddress())
+          .value_or("unknown");
+  socket_node_ = MakeRefCounted<channelz::SocketNode>(
+      grpc_event_engine::experimental::ResolvedAddressToString(
+          control_endpoint.GetLocalAddress())
+          .value_or("unknown"),
+      peer_string, absl::StrCat("chaotic-good server", peer_string),
+      args.GetObjectRef<channelz::SocketNode::Security>());
   auto transport = MakeRefCounted<ChaoticGoodTransport>(
       std::move(control_endpoint), config.TakePendingDataEndpoints(),
       event_engine_,
       args.GetObjectRef<GlobalStatsPluginRegistry::StatsPluginGroup>(),
-      config.MakeTransportOptions(), false);
+      config.MakeTransportOptions(), false, socket_node_);
   auto party_arena = SimpleArenaAllocator(0)->MakeArena();
   party_arena->SetContext<grpc_event_engine::experimental::EventEngine>(
       event_engine_.get());
@@ -364,8 +374,8 @@ ChaoticGoodServerTransport::ChaoticGoodServerTransport(
 
 void ChaoticGoodServerTransport::SetCallDestination(
     RefCountedPtr<UnstartedCallDestination> call_destination) {
-  CHECK(call_destination_ == nullptr);
-  CHECK(call_destination != nullptr);
+  GRPC_CHECK(call_destination_ == nullptr);
+  GRPC_CHECK(call_destination != nullptr);
   call_destination_ = call_destination;
   got_acceptor_.Set();
 }
@@ -502,5 +512,13 @@ void ChaoticGoodServerTransport::PerformOp(grpc_transport_op* op) {
   ExecCtx::Run(DEBUG_LOCATION, op->on_consumed, absl::OkStatus());
 }
 
+void ChaoticGoodServerTransport::ChannelzDataSource::AddData(
+    channelz::DataSink sink) {
+  transport_->party_->ExportToChannelz("party", sink);
+  MutexLock lock(&transport_->mu_);
+  sink.AddData(
+      "client_transport",
+      channelz::PropertyList().Set("streams", transport_->stream_map_.size()));
+}
 }  // namespace chaotic_good_legacy
 }  // namespace grpc_core
