@@ -19,20 +19,17 @@
 #include "src/core/lib/iomgr/combiner.h"
 
 #include <assert.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/port_platform.h>
 #include <inttypes.h>
 #include <string.h>
 
 #include "absl/log/check.h"
 #include "absl/log/log.h"
-
-#include <grpc/support/alloc.h>
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/experiments/experiments.h"
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/mpscq.h"
-#include "src/core/lib/iomgr/executor.h"
 #include "src/core/lib/iomgr/iomgr_internal.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/mpscq.h"
 
 #define STATE_UNORPHANED 1
 #define STATE_ELEM_COUNT_LOW_BIT 2
@@ -162,7 +159,6 @@ static void queue_offload(grpc_core::Combiner* lock) {
   gpr_atm_no_barrier_store(&lock->initiating_exec_ctx_or_null, 1);
   GRPC_TRACE_LOG(combiner, INFO) << "C:" << lock << " queue_offload";
   lock->event_engine->Run([lock] {
-    grpc_core::ApplicationCallbackExecCtx callback_exec_ctx;
     grpc_core::ExecCtx exec_ctx(0);
     push_last_on_exec_ctx(lock);
     exec_ctx.Flush();
@@ -170,6 +166,8 @@ static void queue_offload(grpc_core::Combiner* lock) {
 }
 
 bool grpc_combiner_continue_exec_ctx() {
+  GRPC_LATENT_SEE_SCOPE("grpc_combiner_continue_exec_ctx");
+
   grpc_core::Combiner* lock =
       grpc_core::ExecCtx::Get()->combiner_data()->active_combiner;
   if (lock == nullptr) {
@@ -190,7 +188,7 @@ bool grpc_combiner_continue_exec_ctx() {
   // as soon as possible
   if (contended && grpc_core::ExecCtx::Get()->IsReadyToFinish()) {
     // this execution context wants to move on: schedule remaining work to be
-    // picked up on the executor
+    // picked up on the EventEngine
     queue_offload(lock);
     return true;
   }
