@@ -16,31 +16,28 @@
 //
 //
 
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/iomgr/iomgr.h"
 
+#include <grpc/support/alloc.h>
+#include <grpc/support/port_platform.h>
+#include <grpc/support/string_util.h>
+#include <grpc/support/sync.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
-#include <grpc/support/string_util.h>
-#include <grpc/support/sync.h>
-
-#include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/gpr/string.h"
-#include "src/core/lib/gpr/useful.h"
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/thd.h"
+#include "absl/log/log.h"
+#include "src/core/config/config_vars.h"
 #include "src/core/lib/iomgr/buffer_list.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
-#include "src/core/lib/iomgr/executor.h"
 #include "src/core/lib/iomgr/internal_errqueue.h"
 #include "src/core/lib/iomgr/iomgr_internal.h"
 #include "src/core/lib/iomgr/timer.h"
 #include "src/core/lib/iomgr/timer_manager.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/string.h"
+#include "src/core/util/thd.h"
+#include "src/core/util/useful.h"
 
 static gpr_mu g_mu;
 static gpr_cv g_rcv;
@@ -55,7 +52,6 @@ void grpc_iomgr_init() {
   g_shutdown = 0;
   gpr_mu_init(&g_mu);
   gpr_cv_init(&g_rcv);
-  grpc_core::Executor::InitAll();
   g_root_object.next = g_root_object.prev = &g_root_object;
   g_root_object.name = const_cast<char*>("root");
   grpc_iomgr_platform_init();
@@ -83,7 +79,7 @@ size_t grpc_iomgr_count_objects_for_testing(void) {
 static void dump_objects(const char* kind) {
   grpc_iomgr_object* obj;
   for (obj = g_root_object.next; obj != &g_root_object; obj = obj->next) {
-    gpr_log(GPR_DEBUG, "%s OBJECT: %s %p", kind, obj->name, obj);
+    VLOG(2) << kind << " OBJECT: " << obj->name << " " << obj;
   }
 }
 
@@ -103,9 +99,8 @@ void grpc_iomgr_shutdown() {
               gpr_time_sub(gpr_now(GPR_CLOCK_REALTIME), last_warning_time),
               gpr_time_from_seconds(1, GPR_TIMESPAN)) >= 0) {
         if (g_root_object.next != &g_root_object) {
-          gpr_log(GPR_DEBUG,
-                  "Waiting for %" PRIuPTR " iomgr objects to be destroyed",
-                  count_objects());
+          VLOG(2) << "Waiting for " << count_objects()
+                  << " iomgr objects to be destroyed";
         }
         last_warning_time = gpr_now(GPR_CLOCK_REALTIME);
       }
@@ -119,11 +114,9 @@ void grpc_iomgr_shutdown() {
       }
       if (g_root_object.next != &g_root_object) {
         if (grpc_iomgr_abort_on_leaks()) {
-          gpr_log(GPR_DEBUG,
-                  "Failed to free %" PRIuPTR
-                  " iomgr objects before shutdown deadline: "
-                  "memory leaks are likely",
-                  count_objects());
+          VLOG(2) << "Failed to free " << count_objects()
+                  << " iomgr objects before shutdown deadline: "
+                  << "memory leaks are likely";
           dump_objects("LEAKED");
           abort();
         }
@@ -134,11 +127,9 @@ void grpc_iomgr_shutdown() {
           if (gpr_time_cmp(gpr_now(GPR_CLOCK_REALTIME), shutdown_deadline) >
               0) {
             if (g_root_object.next != &g_root_object) {
-              gpr_log(GPR_DEBUG,
-                      "Failed to free %" PRIuPTR
-                      " iomgr objects before shutdown deadline: "
-                      "memory leaks are likely",
-                      count_objects());
+              VLOG(2) << "Failed to free " << count_objects()
+                      << " iomgr objects before shutdown deadline: "
+                      << "memory leaks are likely";
               dump_objects("LEAKED");
             }
             break;
@@ -149,7 +140,6 @@ void grpc_iomgr_shutdown() {
     gpr_mu_unlock(&g_mu);
     grpc_timer_list_shutdown();
     grpc_core::ExecCtx::Get()->Flush();
-    grpc_core::Executor::ShutdownAll();
   }
 
   // ensure all threads have left g_mu
