@@ -16,25 +16,20 @@
 //
 //
 
-#include <inttypes.h>
-
 #include <grpc/support/port_platform.h>
+#include <inttypes.h>
 
 #include "src/core/lib/iomgr/port.h"
 
 #ifdef GRPC_WINSOCK_SOCKET
 
-#include "absl/log/check.h"
-
 #include <grpc/event_engine/endpoint_config.h>
 #include <grpc/slice_buffer.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 #include <grpc/support/log_windows.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/event_engine/shim.h"
-#include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/iomgr/event_engine_shims/tcp_client.h"
 #include "src/core/lib/iomgr/iocp_windows.h"
 #include "src/core/lib/iomgr/sockaddr.h"
@@ -45,6 +40,8 @@
 #include "src/core/lib/iomgr/timer.h"
 #include "src/core/lib/resource_quota/api.h"
 #include "src/core/lib/slice/slice_internal.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/grpc_check.h"
 
 using ::grpc_event_engine::experimental::EndpointConfig;
 
@@ -85,7 +82,7 @@ static void on_alarm(void* acp, grpc_error_handle /* error */) {
 static void on_connect(void* acp, grpc_error_handle error) {
   async_connect* ac = (async_connect*)acp;
   grpc_endpoint** ep = ac->endpoint;
-  CHECK(*ep == NULL);
+  GRPC_CHECK(*ep == NULL);
   grpc_closure* on_done = ac->on_done;
 
   gpr_mu_lock(&ac->mu);
@@ -99,12 +96,12 @@ static void on_connect(void* acp, grpc_error_handle error) {
 
   if (error.ok()) {
     if (socket != NULL) {
-      DWORD transfered_bytes = 0;
+      DWORD transferred_bytes = 0;
       DWORD flags;
       BOOL wsa_success =
           WSAGetOverlappedResult(socket->socket, &socket->write_info.overlapped,
-                                 &transfered_bytes, FALSE, &flags);
-      CHECK_EQ(transfered_bytes, 0);
+                                 &transferred_bytes, FALSE, &flags);
+      GRPC_CHECK_EQ(transferred_bytes, 0);
       if (!wsa_success) {
         error = GRPC_WSA_ERROR(WSAGetLastError(), "ConnectEx");
         closesocket(socket->socket);
@@ -245,11 +242,9 @@ static int64_t tcp_connect(grpc_closure* on_done, grpc_endpoint** endpoint,
   return 0;
 
 failure:
-  CHECK(!error.ok());
-  grpc_error_handle final_error = grpc_error_set_str(
-      GRPC_ERROR_CREATE_REFERENCING("Failed to connect", &error, 1),
-      grpc_core::StatusStrProperty::kTargetAddress,
-      addr_uri.ok() ? *addr_uri : addr_uri.status().ToString());
+  GRPC_CHECK(!error.ok());
+  grpc_error_handle final_error =
+      GRPC_ERROR_CREATE_REFERENCING("Failed to connect", &error, 1);
   if (socket != NULL) {
     grpc_winsocket_destroy(socket);
   } else if (sock != INVALID_SOCKET) {
