@@ -17,47 +17,40 @@
 //
 #include "src/core/lib/surface/completion_queue.h"
 
+#include <grpc/event_engine/event_engine.h>
 #include <grpc/grpc.h>
 #include <grpc/support/alloc.h>
 #include <grpc/support/atm.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/sync.h>
 #include <grpc/support/time.h>
-#include <inttypes.h>
 #include <stdio.h>
 
 #include <algorithm>
 #include <atomic>
-#include <new>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "absl/log/check.h"
 #include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/str_join.h"
-#include "src/core/lib/event_engine/default_event_engine.h"
+#include "src/core/lib/event_engine/shim.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
-#include "src/core/lib/iomgr/executor.h"
-#include "src/core/lib/iomgr/iomgr.h"
 #include "src/core/lib/iomgr/pollset.h"
 #include "src/core/lib/surface/event_string.h"
 #include "src/core/telemetry/stats.h"
 #include "src/core/telemetry/stats_data.h"
 #include "src/core/util/atomic_utils.h"
 #include "src/core/util/debug_location.h"
+#include "src/core/util/grpc_check.h"
 #include "src/core/util/ref_counted.h"
 #include "src/core/util/spinlock.h"
 #include "src/core/util/status_helper.h"
 #include "src/core/util/time.h"
 
-#ifdef GPR_WINDOWS
-#include "src/core/lib/experiments/experiments.h"
-#endif
-
 namespace {
 
 // Specifies a cq thread local cache.
@@ -73,7 +66,7 @@ struct plucker {
   void* tag;
 };
 struct cq_poller_vtable {
-  bool can_get_pollset;
+  bool (*can_get_pollset)();
   bool can_listen;
   size_t (*size)(void);
   void (*init)(grpc_pollset* pollset, gpr_mu** mu);
@@ -173,7 +166,7 @@ grpc_error_handle non_polling_poller_kick(
 
 void non_polling_poller_shutdown(grpc_pollset* pollset, grpc_closure* closure) {
   non_polling_poller* p = reinterpret_cast<non_polling_poller*>(pollset);
-  CHECK_NE(closure, nullptr);
+  GRPC_CHECK_NE(closure, nullptr);
   p->shutdown = closure;
   if (p->root == nullptr) {
     grpc_core::ExecCtx::Run(DEBUG_LOCATION, closure, absl::OkStatus());
@@ -186,16 +179,83 @@ void non_polling_poller_shutdown(grpc_pollset* pollset, grpc_closure* closure) {
   }
 }
 
+bool non_polling_poller_can_never_get_pollset() { return false; }
+
+bool maybe_event_engine_poller_can_get_pollset() {
+  return !grpc_event_engine::experimental::UsePollsetAlternative();
+}
+
+// maybe_event_engine*: If iomgr isn't relied upon for polling, then we can use
+// the non-polling poller implementations below.
+size_t maybe_event_engine_poller_size(void) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    return non_polling_poller_size();
+  } else {
+    return grpc_pollset_size();
+  }
+}
+
+void maybe_event_engine_poller_init(grpc_pollset* pollset, gpr_mu** mu) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    non_polling_poller_init(pollset, mu);
+  } else {
+    grpc_pollset_init(pollset, mu);
+  }
+}
+
+void maybe_event_engine_poller_destroy(grpc_pollset* pollset) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    non_polling_poller_destroy(pollset);
+  } else {
+    grpc_pollset_destroy(pollset);
+  }
+}
+
+grpc_error_handle maybe_event_engine_pollset_work(
+    grpc_pollset* pollset, grpc_pollset_worker** worker,
+    grpc_core::Timestamp deadline) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    return non_polling_poller_work(pollset, worker, deadline);
+  } else {
+    return grpc_pollset_work(pollset, worker, deadline);
+  }
+}
+
+grpc_error_handle maybe_event_engine_poller_kick(
+    grpc_pollset* pollset, grpc_pollset_worker* specific_worker) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    return non_polling_poller_kick(pollset, specific_worker);
+  } else {
+    return grpc_pollset_kick(pollset, specific_worker);
+  }
+}
+
+void maybe_event_engine_poller_shutdown(grpc_pollset* pollset,
+                                        grpc_closure* closure) {
+  if (grpc_event_engine::experimental::UsePollsetAlternative()) {
+    non_polling_poller_shutdown(pollset, closure);
+  } else {
+    grpc_pollset_shutdown(pollset, closure);
+  }
+}
+
+// TODO(hork): Once EE is the default everywhere, the only variations will be
+// the bool args controlling whether the poller can get a pollset and whether it
+// can listen.
 const cq_poller_vtable g_poller_vtable_by_poller_type[] = {
     // GRPC_CQ_DEFAULT_POLLING
-    {true, true, grpc_pollset_size, grpc_pollset_init, grpc_pollset_kick,
-     grpc_pollset_work, grpc_pollset_shutdown, grpc_pollset_destroy},
+    {maybe_event_engine_poller_can_get_pollset, true,
+     maybe_event_engine_poller_size, maybe_event_engine_poller_init,
+     maybe_event_engine_poller_kick, maybe_event_engine_pollset_work,
+     maybe_event_engine_poller_shutdown, maybe_event_engine_poller_destroy},
     // GRPC_CQ_NON_LISTENING
-    {true, false, grpc_pollset_size, grpc_pollset_init, grpc_pollset_kick,
-     grpc_pollset_work, grpc_pollset_shutdown, grpc_pollset_destroy},
+    {maybe_event_engine_poller_can_get_pollset, false,
+     maybe_event_engine_poller_size, maybe_event_engine_poller_init,
+     maybe_event_engine_poller_kick, maybe_event_engine_pollset_work,
+     maybe_event_engine_poller_shutdown, maybe_event_engine_poller_destroy},
     // GRPC_CQ_NON_POLLING
-    {false, false, non_polling_poller_size, non_polling_poller_init,
-     non_polling_poller_kick, non_polling_poller_work,
+    {non_polling_poller_can_never_get_pollset, false, non_polling_poller_size,
+     non_polling_poller_init, non_polling_poller_kick, non_polling_poller_work,
      non_polling_poller_shutdown, non_polling_poller_destroy},
 };
 
@@ -251,7 +311,7 @@ class CqEventQueue {
 
 struct cq_next_data {
   ~cq_next_data() {
-    CHECK_EQ(queue.num_items(), 0);
+    GRPC_CHECK_EQ(queue.num_items(), 0);
 #ifndef NDEBUG
     if (pending_events.load(std::memory_order_acquire) != 0) {
       LOG(ERROR) << "Destroying CQ without draining it fully.";
@@ -281,7 +341,8 @@ struct cq_pluck_data {
   }
 
   ~cq_pluck_data() {
-    CHECK(completed_head.next == reinterpret_cast<uintptr_t>(&completed_head));
+    GRPC_CHECK(completed_head.next ==
+               reinterpret_cast<uintptr_t>(&completed_head));
 #ifndef NDEBUG
     if (pending_events.load(std::memory_order_acquire) != 0) {
       LOG(ERROR) << "Destroying CQ without draining it fully.";
@@ -650,7 +711,7 @@ static void cq_check_tag(grpc_completion_queue* cq, void* tag, bool lock_cq) {
     gpr_mu_unlock(cq->mu);
   }
 
-  CHECK(found);
+  GRPC_CHECK(found);
 }
 #else
 static void cq_check_tag(grpc_completion_queue* /*cq*/, void* /*tag*/,
@@ -816,16 +877,12 @@ static void cq_end_op_for_pluck(
   }
 }
 
-static void functor_callback(void* arg, grpc_error_handle error) {
-  auto* functor = static_cast<grpc_completion_queue_functor*>(arg);
-  functor->functor_run(functor, error.ok());
-}
-
 // Complete an event on a completion queue of type GRPC_CQ_CALLBACK
 static void cq_end_op_for_callback(
     grpc_completion_queue* cq, void* tag, grpc_error_handle error,
     void (*done)(void* done_arg, grpc_cq_completion* storage), void* done_arg,
     grpc_cq_completion* storage, bool internal) {
+  (void)internal;
   cq_callback_data* cqd = static_cast<cq_callback_data*> DATA_FROM_CQ(cq);
 
   if (GRPC_TRACE_FLAG_ENABLED(api) ||
@@ -851,32 +908,11 @@ static void cq_end_op_for_callback(
   }
 
   auto* functor = static_cast<grpc_completion_queue_functor*>(tag);
-  if (grpc_core::IsEventEngineApplicationCallbacksEnabled()) {
-    // Run the callback on EventEngine threads.
-    cqd->event_engine->Run(
-        [engine = cqd->event_engine, functor, ok = error.ok()]() {
-          grpc_core::ExecCtx exec_ctx;
-          (*functor->functor_run)(functor, ok);
-        });
-    return;
-  }
-  // If possible, schedule the callback onto an existing thread-local
-  // ApplicationCallbackExecCtx, which is a work queue. This is possible for:
-  // 1. The callback is internally-generated and there is an ACEC available
-  // 2. The callback is marked inlineable and there is an ACEC available
-  // 3. We are already running in a background poller thread (which always has
-  //    an ACEC available at the base of the stack).
-  if (((internal || functor->inlineable) &&
-       grpc_core::ApplicationCallbackExecCtx::Available()) ||
-      grpc_iomgr_is_any_background_poller_thread()) {
-    grpc_core::ApplicationCallbackExecCtx::Enqueue(functor, (error.ok()));
-    return;
-  }
-
-  // Schedule the callback on a closure if not internal or triggered
-  // from a background poller thread.
-  grpc_core::Executor::Run(
-      GRPC_CLOSURE_CREATE(functor_callback, functor, nullptr), error);
+  cqd->event_engine->Run(
+      [engine = cqd->event_engine, functor, ok = error.ok()]() {
+        grpc_core::ExecCtx exec_ctx;
+        (*functor->functor_run)(functor, ok);
+      });
 }
 
 void grpc_cq_end_op(grpc_completion_queue* cq, void* tag,
@@ -898,15 +934,14 @@ struct cq_is_finished_arg {
 class ExecCtxNext : public grpc_core::ExecCtx {
  public:
   explicit ExecCtxNext(void* arg)
-      : ExecCtx(0, GRPC_LATENT_SEE_METADATA("ExecCtx for CqNext")),
-        check_ready_to_finish_arg_(arg) {}
+      : ExecCtx(0), check_ready_to_finish_arg_(arg) {}
 
   bool CheckReadyToFinish() override {
     cq_is_finished_arg* a =
         static_cast<cq_is_finished_arg*>(check_ready_to_finish_arg_);
     grpc_completion_queue* cq = a->cq;
     cq_next_data* cqd = static_cast<cq_next_data*> DATA_FROM_CQ(cq);
-    CHECK_EQ(a->stolen_completion, nullptr);
+    GRPC_CHECK_EQ(a->stolen_completion, nullptr);
 
     intptr_t current_last_seen_things_queued_ever =
         cqd->things_queued_ever.load(std::memory_order_relaxed);
@@ -960,7 +995,7 @@ static grpc_event cq_next(grpc_completion_queue* cq, gpr_timespec deadline,
       << ", tv_nsec: " << deadline.tv_nsec
       << ", clock_type: " << (int)deadline.clock_type
       << " }, reserved=" << reserved << ")";
-  CHECK(!reserved);
+  GRPC_CHECK(!reserved);
 
   dump_pending_tags(cq);
 
@@ -1066,7 +1101,7 @@ static grpc_event cq_next(grpc_completion_queue* cq, gpr_timespec deadline,
   GRPC_SURFACE_TRACE_RETURNED_EVENT(cq, &ret);
   GRPC_CQ_INTERNAL_UNREF(cq, "next");
 
-  CHECK_EQ(is_finished_arg.stolen_completion, nullptr);
+  GRPC_CHECK_EQ(is_finished_arg.stolen_completion, nullptr);
 
   return ret;
 }
@@ -1080,8 +1115,8 @@ static grpc_event cq_next(grpc_completion_queue* cq, gpr_timespec deadline,
 static void cq_finish_shutdown_next(grpc_completion_queue* cq) {
   cq_next_data* cqd = static_cast<cq_next_data*> DATA_FROM_CQ(cq);
 
-  CHECK(cqd->shutdown_called);
-  CHECK_EQ(cqd->pending_events.load(std::memory_order_relaxed), 0);
+  GRPC_CHECK(cqd->shutdown_called);
+  GRPC_CHECK_EQ(cqd->pending_events.load(std::memory_order_relaxed), 0);
 
   cq->poller_vtable->shutdown(POLLSET_FROM_CQ(cq), &cq->pollset_shutdown_done);
 }
@@ -1146,8 +1181,7 @@ static void del_plucker(grpc_completion_queue* cq, void* tag,
 class ExecCtxPluck : public grpc_core::ExecCtx {
  public:
   explicit ExecCtxPluck(void* arg)
-      : ExecCtx(0, GRPC_LATENT_SEE_METADATA("ExecCtx for CqPluck")),
-        check_ready_to_finish_arg_(arg) {}
+      : ExecCtx(0), check_ready_to_finish_arg_(arg) {}
 
   bool CheckReadyToFinish() override {
     cq_is_finished_arg* a =
@@ -1155,7 +1189,7 @@ class ExecCtxPluck : public grpc_core::ExecCtx {
     grpc_completion_queue* cq = a->cq;
     cq_pluck_data* cqd = static_cast<cq_pluck_data*> DATA_FROM_CQ(cq);
 
-    CHECK_EQ(a->stolen_completion, nullptr);
+    GRPC_CHECK_EQ(a->stolen_completion, nullptr);
     gpr_atm current_last_seen_things_queued_ever =
         cqd->things_queued_ever.load(std::memory_order_relaxed);
     if (current_last_seen_things_queued_ever !=
@@ -1203,7 +1237,7 @@ static grpc_event cq_pluck(grpc_completion_queue* cq, void* tag,
         << ", clock_type: " << (int)deadline.clock_type
         << " }, reserved=" << reserved << ")";
   }
-  CHECK(!reserved);
+  GRPC_CHECK(!reserved);
 
   dump_pending_tags(cq);
 
@@ -1293,7 +1327,7 @@ done:
   GRPC_SURFACE_TRACE_RETURNED_EVENT(cq, &ret);
   GRPC_CQ_INTERNAL_UNREF(cq, "pluck");
 
-  CHECK_EQ(is_finished_arg.stolen_completion, nullptr);
+  GRPC_CHECK_EQ(is_finished_arg.stolen_completion, nullptr);
 
   return ret;
 }
@@ -1306,8 +1340,8 @@ grpc_event grpc_completion_queue_pluck(grpc_completion_queue* cq, void* tag,
 static void cq_finish_shutdown_pluck(grpc_completion_queue* cq) {
   cq_pluck_data* cqd = static_cast<cq_pluck_data*> DATA_FROM_CQ(cq);
 
-  CHECK(cqd->shutdown_called);
-  CHECK(!cqd->shutdown.load(std::memory_order_relaxed));
+  GRPC_CHECK(cqd->shutdown_called);
+  GRPC_CHECK(!cqd->shutdown.load(std::memory_order_relaxed));
   cqd->shutdown.store(true, std::memory_order_relaxed);
 
   cq->poller_vtable->shutdown(POLLSET_FROM_CQ(cq), &cq->pollset_shutdown_done);
@@ -1343,27 +1377,14 @@ static void cq_finish_shutdown_callback(grpc_completion_queue* cq) {
   cq_callback_data* cqd = static_cast<cq_callback_data*> DATA_FROM_CQ(cq);
   auto* callback = cqd->shutdown_callback;
 
-  CHECK(cqd->shutdown_called);
+  GRPC_CHECK(cqd->shutdown_called);
 
   cq->poller_vtable->shutdown(POLLSET_FROM_CQ(cq), &cq->pollset_shutdown_done);
 
-  if (grpc_core::IsEventEngineApplicationCallbacksEnabled()) {
-    cqd->event_engine->Run([engine = cqd->event_engine, callback]() {
-      grpc_core::ExecCtx exec_ctx;
-      callback->functor_run(callback, /*true=*/1);
-    });
-    return;
-  }
-  if (grpc_iomgr_is_any_background_poller_thread()) {
-    grpc_core::ApplicationCallbackExecCtx::Enqueue(callback, true);
-    return;
-  }
-
-  // Schedule the callback on a closure if not internal or triggered
-  // from a background poller thread.
-  grpc_core::Executor::Run(
-      GRPC_CLOSURE_CREATE(functor_callback, callback, nullptr),
-      absl::OkStatus());
+  cqd->event_engine->Run([engine = cqd->event_engine, callback]() {
+    grpc_core::ExecCtx exec_ctx;
+    callback->functor_run(callback, /*true=*/1);
+  });
 }
 
 static void cq_shutdown_callback(grpc_completion_queue* cq) {
@@ -1395,7 +1416,6 @@ static void cq_shutdown_callback(grpc_completion_queue* cq) {
 // Shutdown simply drops a ref that we reserved at creation time; if we drop
 // to zero here, then enter shutdown mode and wake up any waiters
 void grpc_completion_queue_shutdown(grpc_completion_queue* cq) {
-  grpc_core::ApplicationCallbackExecCtx callback_exec_ctx;
   grpc_core::ExecCtx exec_ctx;
   GRPC_TRACE_LOG(api, INFO)
       << "grpc_completion_queue_shutdown(cq=" << cq << ")";
@@ -1411,7 +1431,7 @@ void grpc_completion_queue_destroy(grpc_completion_queue* cq) {
 }
 
 grpc_pollset* grpc_cq_pollset(grpc_completion_queue* cq) {
-  return cq->poller_vtable->can_get_pollset ? POLLSET_FROM_CQ(cq) : nullptr;
+  return cq->poller_vtable->can_get_pollset() ? POLLSET_FROM_CQ(cq) : nullptr;
 }
 
 bool grpc_cq_can_listen(grpc_completion_queue* cq) {
